@@ -1,3 +1,58 @@
+#' @title compute Ball Divergence statistic
+#' @description wrapper C function which compute Ball Divergence via limit distribution
+#' @inheritParams bd.test
+#' @noRd
+bcov_limit_wrap_c <- function(x, y, num, distance, num.threads) {
+  if (!is.null(y)) {
+    x <- list(x, y)
+  }
+  N <- as.integer(num)
+  distance <- as.integer(distance)
+  num.threads <- as.integer(num.threads)
+  
+  bdd_xy_eigen <- matrix(data = 1, ncol = num, nrow = num)
+  for (i in 1:length(x)) {
+    xy <- as.double(x[[i]])
+    bdd_xy <- double((num + 1) * num / 2)
+    res <- .C("bdd_matrix_bias", bdd_xy, xy, N, num.threads)
+    rm(bdd_xy); gc(reset = TRUE, verbose = FALSE)
+    bdd_xy <- matrix(0, nrow = num, ncol = num)
+    bdd_xy[lower.tri(bdd_xy, diag = TRUE)] <- res[[1]]
+    bdd_xy <- bdd_xy + t(bdd_xy)
+    diag(bdd_xy) <- diag(bdd_xy) / 2
+    bdd_xy_eigen <- bdd_xy_eigen * center_bdd_matrix(bdd_xy)
+  }
+  
+  eigenvalue <- eigen(bdd_xy_eigen, only.values = TRUE, symmetric = TRUE)$values
+  eigenvalue <- eigenvalue[eigenvalue > 0] / num
+  eigenvalue
+}
+
+#' @title compute Ball Divergence statistic
+#' @description wrapper C function which compute Ball Divergence via limit distribution
+#' @inheritParams bd.test
+#' @noRd
+bd_limit_wrap_c <- function(xy, size, distance, num.threads) {
+  xy <- as.double(xy)
+  n1 <- as.integer(size[1])
+  n2 <- as.integer(size[2])
+  distance <- as.integer(distance)
+  num.threads <- as.integer(num.threads)
+  
+  num <- as.integer(sum(size))
+  bdd_xy <- double((num + 1) * num / 2)
+  res <- .C("bdd_matrix_bias_two_group", bdd_xy, xy, n1, n2, num.threads)
+  bdd_xy <- matrix(0, nrow = num, ncol = num)
+  bdd_xy[lower.tri(bdd_xy, diag = TRUE)] <- res[[1]]
+  bdd_xy <- bdd_xy + t(bdd_xy)
+  diag(bdd_xy) <- diag(bdd_xy) / 2
+  
+  bdd_xy <- center_bdd_matrix(bdd_xy)
+  eigenvalue <- eigen(bdd_xy, only.values = TRUE, symmetric = TRUE)$values
+  eigenvalue <- eigenvalue[eigenvalue > 0] / num
+  2 * eigenvalue
+}
+
 #' @title compute Ball Divergence statistic
 #' @description wrapper C function which compute Ball Divergence
 #' @inheritParams bd.test
@@ -45,24 +100,35 @@ bd_test_wrap_c <- function(xy, size, num.permutations, weight, distance, num.thr
   size <- as.integer(size)
   N <- as.integer(sum(size))
   res <- .C("bd_test", bd, p_value, xy, size, N, K, distance, r, num.threads)
-  #
-  stat_name_bd <- c("bd", "wbd")
-  stat_name_kbd <- c("kbd.sum", "wkbd.sum", "kbd.max", "wkbd.max", "kbd.maxsum", "wkbd.maxsum")
+
   bd <- res[[1]]
   p_value <- res[[2]]
   if (K == 2) {
-    names(bd) <- stat_name_bd
-    bd <- bd[1]
-    p_value <- p_value[1]
+    names(bd) <- BD_WEIGHT_STATS
   } else {
-    names(bd) <- stat_name_kbd
-    bd <- bd[c(1, 3, 5)]
-    p_value <- p_value[c(1, 3, 5)]
+    names(bd) <- KBD_WEIGHT_STATS
   }
   names(p_value) <- paste0(names(bd), ".pvalue")
-  list('statistic' = bd, 'p.value' = p_value, 
-       'info' = list('N' = N, 'K' = K, 'size' = size, 
-                     'weight' = as.logical(weight), 'num.permutations' = num.permutations))
+  
+  if (weight == BD_WEIGHT_TYPE[1]) {
+    if (K == 2) {
+      index <- 1
+    } else {
+      index <- c(1, 3, 5)
+    }
+  } else {
+    if (K == 2) {
+      index <- 2
+    } else {
+      index <- c(2, 4, 6)
+    }
+  }
+  return_bd <- bd[index]
+  return_p_value <- p_value[index]
+  
+  list('statistic' = return_bd, 'p.value' = return_p_value, 
+       'info' = list('statistic' = bd, "p.value" = p_value, 'N' = N, 'K' = K, 'size' = size, 
+                     'weight' = weight, 'num.permutations' = num.permutations))
 }
 
 
@@ -190,25 +256,62 @@ kbcov_test_wrap_c <- function(x, K, n, num.permutations, distance, num.threads)
 #' @useDynLib Ball, .registration = TRUE
 #' @noRd
 #' 
-apply_bcor_wrap <- function(x, y, n, p, distance, weight, method, num.threads) {
-  bcor_stat <- as.double(numeric(3*p))
+apply_bcor_wrap <- function(x, y, n, p, distance, weight, method, num.threads, category) {
+  p_all <- ncol(x)
+  if (length(category) != 0) {
+    x_category <- x[, category, drop = FALSE]
+    x <- x[, -category, drop = FALSE]
+  } else {
+    x_category <- matrix(0, nrow = 0, ncol = 0)
+  }
+  p_continuous <- ncol(x)
+  p_category <- ncol(x_category)
+  
+  if (distance) {
+    p <- as.integer(0)
+  } else {
+    p <- as.integer(p)
+  }
   y <- as.double(y)
-  x <- as.vector(x)
-  x_number <- rep(1, p)
-  f_number <- as.integer(p)
-  size_num <- 2
   num <- as.integer(n)
-  nthread <- integer(1)
-  p <- as.integer(1)
-  k <- as.integer(1)
-  dst_y <- as.integer(distance)
   dst_x <- as.integer(0)
   nth <- as.integer(num.threads)
-  #
-  res <- .C("bcor_test", bcor_stat, y, x, x_number, f_number, size_num, num, p, k, dst_y, dst_x, nth)[[1]]
-  bcor_stat <- matrix(res, ncol = 3, byrow = TRUE)
+  dst_y <- as.integer(distance)
+  if (p_continuous != 0) {
+    bcor_stat1 <- as.double(numeric(3 * p_continuous))
+    x <- as.double(as.vector(x))
+    x_number <- as.integer(rep(1, p_continuous))
+    f_number <- as.integer(p_continuous)
+    k <- as.integer(1)
+    #
+    res <- .C("bcor_test", bcor_stat1, y, x, x_number, f_number, num, p, k, dst_y, dst_x, nth)[[1]]
+    bcor_stat1 <- matrix(res, ncol = 3, byrow = TRUE)
+  }
+  
+  if (p_category != 0) {
+    bcor_stat2 <- as.double(numeric(3 * p_category))
+    x <- as.double(as.vector(x_category))
+    x_number <- as.integer(rep(1, p_category))
+    f_number <- as.integer(p_category)
+    k <- as.integer(2)
+    #
+    res <- .C("bcor_test", bcor_stat2, y, x, x_number, f_number, num, p, k, dst_y, dst_x, nth)[[1]]
+    bcor_stat2 <- matrix(res, ncol = 3, byrow = TRUE)
+    
+    if (p_continuous == 0) {
+      bcor_stat <- bcor_stat2
+    } else {
+      bcor_stat <- matrix(nrow = p_all, ncol = 3)
+      bcor_stat[-category, ] <- bcor_stat1
+      bcor_stat[category, ] <- bcor_stat2 
+    }
+  } else {
+    bcor_stat <- bcor_stat1
+  }
+  
   colnames(bcor_stat) <- BCOR_WEIGHT_STATS
   screening_bcor_stat <- select_ball_stat(bcor_stat, weight = weight, fun_name = "bcorsis")
+  
   if (method %in% c("interaction", "standard"))
   {
     return(list(bcor_stat, screening_bcor_stat))
